unit fMain;

interface

uses
  Windows, Messages, SysUtils,
  Variants,
  Classes, Graphics, Controls, Forms,
  Dialogs, atDiagram, ExtCtrls, DiagramExtra, StdCtrls;

type
  TForm1 = class(TForm)
    DiagramToolBar1: TDiagramToolBar;
    atDiagram1: TatDiagram;
    Splitter1: TSplitter;
    Panel1: TPanel;
    Memo1: TMemo;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    procedure atDiagram1InsertDControl(Sender: TObject;
      ADControl: TDiagramControl);
    procedure atDiagram1DControlDblClick(Sender: TObject;
      ADControl: TDiagramControl);
    procedure Button1Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    procedure CheckBlockLinks(ABlock: TCustomDiagramBlock);
    { Private declarations }
  public
    { Public declarations }
    AnchorBlock, CleanBlock : TDiagramBlock;
  end;

var
  Form1: TForm1;

implementation
uses FlowChartBlocks;

{$R *.dfm}

procedure TForm1.atDiagram1InsertDControl(Sender: TObject;
  ADControl: TDiagramControl);
begin
  {Just put the name of component in the text cell, so we can
   know the names of components visually}
  if ADControl is TCustomDiagramBlock then
    TCustomDiagramBlock(ADControl).Strings.Text := ADControl.Name
  else
  if ADControl is TCustomDiagramLine then
  begin
    With TCustomDiagramLine(ADControl) do
    begin
      if TextCells.Count > 0 then
        TextCells[0].Text := ADControl.Name;

      {In addition, if it's a line, then put an arrow in the target point
       so the user can know the origin and destination of the link}
      TargetArrow.Shape := asSolidArrow;
    end;
  end;
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  i, x, y : integer;
  ThisLine : TDiagramLine;
begin
  AnchorBlock := TDiagramBlock.Create(self);
  with AnchorBlock do
  begin
   Left := atDiagram1.Width * 0.1;
   Top := atDiagram1.Height * 0.1;
   Width := atDiagram1.Width * 0.8;
   Height := atDiagram1.Height * 0.1;
   Diagram := atDiagram1;

   LinkPoints.Add(5,5);
   LinkPoints.Add(50,5);
   LinkPoints.Add(95,5);
  end;

  CleanBlock := TDiagramBlock.Create(self);
  with CleanBlock do
  begin
   Left := atDiagram1.Width * 0.3;
   Top := atDiagram1.Height * 0.3;
   Width := atDiagram1.Width * 0.4;
   Height := atDiagram1.Height * 0.4;
   Diagram := atDiagram1;
  end;

  for i := 1 to 6 do
  begin
    x := round(i * 15);
    y := round(i * 15);
    CleanBlock.LinkPoints.Add(x,y);
  end;

  for i := 0 to 2 do
  begin
    ThisLine := TDiagramLine.Create(atDiagram1.owner);
    with ThisLine do
    begin
      Diagram := atDiagram1;
      SourceLinkPoint.Anchor := AnchorBlock;
      SourceLinkPoint.AnchorIndex := i;
      TargetLinkPoint.Anchor := CleanBlock;
      TargetLinkPoint.AnchorIndex := i * 2;
    end;
  end;
end;

procedure TForm1.Button2Click(Sender: TObject);
var
  i : integer;
begin
  if CleanBlock = nil then exit;

  for i  := CleanBlock.LinkPoints.Count-1  downto 0 do
    if CleanBlock.LinkPoints[i].AnchoredCount = 0 then
      CleanBlock.LinkPoints[i].Visible := false;
  atDiagram1.Invalidate;
end;

procedure TForm1.Button3Click(Sender: TObject);
var
  i : integer;
begin
  if CleanBlock = nil then exit;

  for i  := CleanBlock.LinkPoints.Count-1  downto 0 do
    if CleanBlock.LinkPoints[i].AnchoredCount = 0 then
      CleanBlock.LinkPoints[i].free;
end;

procedure TForm1.CheckBlockLinks(ABlock: TCustomDiagramBlock);

  procedure CheckLink(ALink: TCustomDiagramLine);
  var
    ConnectStr: string;
    ConnectedControl: TDiagramControl;
    ConnectedName: string;
  begin
    {A link has two connection points. We must find out which point has
     the block we are analysing, and which point has the other connected block
     (if any)}
    if ALink.SourceLinkPoint.Anchor = ABlock then
    begin
      ConnectedControl := ALink.TargetLinkPoint.Anchor;
      ConnectStr := '-->';
    end else
    begin
      ConnectedControl := ALink.SourceLinkPoint.Anchor;
      ConnectStr := '<--';
    end;

    if ConnectedControl = nil then
      ConnectedName := 'none'
    else
      ConnectedName := ConnectedControl.Name;
    Memo1.Lines.Add(Format('%s (%s) %s %s',
      [ConnectStr, ALink.Name, ConnectStr, ConnectedName]));
  end;

var
  c: integer;
  d: integer;
begin
  Memo1.Lines.Add(Format('%s links:',[ABlock.Name]));

  {Iterate through each link point (connection point) available in block}
  for c := 0 to ABlock.LinkPoints.Count - 1 do
    {For each link point, iterate through the controls which are anchored to it}
    for d := 0 to ABlock.LinkPoints[c].AnchoredCount - 1 do
    begin
      {If the anchored control is a line, then analise it.
       * for now, only lines can be anchored to a link point, so this
         test is not needed. But in a future version other types of
         controls can be anchored to a link point, so do the test already}
      if ABlock.LinkPoints[c].Anchoreds[d].DControl is TCustomDiagramLine then
        CheckLink(TCustomDiagramLine(ABlock.LinkPoints[c].Anchoreds[d].DControl));
    end;

  Memo1.Lines.Add('---');
end;

procedure TForm1.atDiagram1DControlDblClick(Sender: TObject;
  ADControl: TDiagramControl);
begin
  if ADControl is TCustomDiagramBlock then
    CheckBlockLinks(TCustomDiagramBlock(ADControl));
end;

end.
